// Author: Hugo Pointcheval
// Email: git@pcl.ovh
// -----
// File: sign_up_form.dart
// Created Date: 19/08/2022 14:41:08
// Last Modified: Fri Nov 11 2022
// -----
// Copyright (c) 2022

import 'package:example_router/core/constants/form_field.dart';
import 'package:flutter/material.dart' hide FormField;
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class _EmailInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<SignUpCubit<int>>(
      field: AuthFormField.email,
      builder: ((context, cubit, state, field, inputValid) {
        return TextField(
          onChanged: (email) => cubit.emailChanged(email),
          keyboardType: TextInputType.emailAddress,
          decoration: InputDecoration(
            labelText: 'Email',
            helperText: '',
            errorText: !inputValid ? 'Invalid email' : null,
          ),
        );
      }),
    );
  }
}

class _PasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<SignUpCubit<int>>(
      field: AuthFormField.password,
      builder: ((context, cubit, state, field, inputValid) {
        return TextField(
          onChanged: (pwd) {
            cubit.passwordChanged(pwd);
            cubit.dataChanged(
                AppFormField.confirmedPassword,
                ConfirmedPassword.dirty(
                    password: pwd,
                    value: state.form
                        .valueOf<String?>(AppFormField.confirmedPassword)));
          },
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'Password',
            helperText: '',
            errorText: !inputValid ? 'Invalid password' : null,
          ),
        );
      }),
    );
  }
}

class _ConfirmPasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<SignUpCubit<int>>(
      field: AppFormField.confirmedPassword,
      builder: ((context, cubit, state, field, inputValid) {
        return TextField(
          onChanged: (pwd) {
            cubit.dataChanged(
              field,
              ConfirmedPassword.dirty(
                  password:
                      state.form.valueOf<String?>(AuthFormField.password) ?? '',
                  value: pwd),
            );
          },
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'Confirm password',
            helperText: '',
            errorText: !inputValid ? 'Passwords do not match' : null,
          ),
        );
      }),
    );
  }
}

class _SignUpButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<SignUpCubit<int>>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: status.isValidated ? () => cubit.submit() : null,
                child: const Text('Sign up'),
              );
      }),
    );
  }
}

class SignUpForm extends StatelessWidget {
  const SignUpForm({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SignUpListener<int>(
        onError: (context, status, errorMessage) =>
            ScaffoldMessenger.of(context)
              ..hideCurrentSnackBar()
              ..showSnackBar(
                SnackBar(content: Text(errorMessage ?? 'Sign Up Failure')),
              ),
        child: SingleChildScrollView(
          child: Column(
            children: [
              _EmailInput(),
              const SizedBox(height: 8),
              _PasswordInput(),
              const SizedBox(height: 8),
              _ConfirmPasswordInput(),
              const SizedBox(height: 16),
              _SignUpButton(),
            ],
          ),
        ));
  }
}
